package com.nsgk.agentcentersdk.entity;

import com.nsgk.agentcentersdk.err.NSErrGlobal;
import com.nsgk.agentcentersdk.err.NSErrno;
import com.nsgk.agentcentersdk.utility.NSArr;
import com.nsgk.agentcentersdk.utility.NSReflect;
import lombok.Data;
import lombok.experimental.Accessors;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.LinkedHashMap;
import java.util.Map;

// 基本实体
@Data
@Accessors(chain = true)
public class NSEntityBase implements Serializable
{
    private static final long serialVersionUID = 1L;

    /** 账套ID */
    protected Long bookId;

    /** 部门id */
    protected Long deptId;

    /** 外部ID */
    protected Long outId;

    protected Map<String, Object> parms;

    protected Map<String, Object> PARMS()
    {
        if(null == parms)
            parms = new LinkedHashMap<>();
        return parms;
    }

    @SuppressWarnings("unchecked")
    public <T extends NSEntityBase> T Parm(String name, Object val)
    {
        PARMS().put(name, val);
        return (T)this;
    }

    @SuppressWarnings("unchecked")
    public <T extends NSEntityBase> T DelParm(String...name)
    {
        if(null != parms && NSArr.IsNotEmpty(name))
        {
            NSArr.Stream(name).forEach(parms::remove);
        }
        return (T)this;
    }

    public boolean HasParm(String name)
    {
        if(null == parms)
            return false;
        return parms.containsKey(name);
    }

    @SuppressWarnings("unchecked")
    public <T> T ParmT(String name, T...val)
    {
        T def = NSArr.DefParm(val);
        if(null == parms)
            return def;
        return (T)parms.getOrDefault(name, def);
    }

    // Set函数为了兼容SDK
    // 不检查类型
    public <T> boolean Set(String name, T val)
    {
        Field field;

        try
        {
            field = NSReflect.FindField(getClass(), name);
            if(null != field)
            {
                if(!field.isAccessible())
                    field.setAccessible(true);
                field.set(this, val);
                return true;
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        return NSErrGlobal.ThrowAndReturn(NSErrno.ERRNO_SYS_INVALID_FIELD, false);
    }

    // 检查类型
    public boolean Set_s(String name, Object val)
    {
        Field field;

        try
        {
            field = getClass().getField(name);
            if(field.getType().isAssignableFrom(val.getClass()))
            {
                return Set(name, val);
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        return NSErrGlobal.ThrowAndReturn(NSErrno.ERRNO_SYS_INVALID_FIELD, false);
    }
}
